// Copyright 2020 Contributors to the Parsec project.
// SPDX-License-Identifier: Apache-2.0

/// This module contains code that deals with the non volatile
/// parts of the tpm.

/// Non volatile storage module.
pub mod storage;
