#[doc = "*Required features: `\"Win32_System_WinRT_Shell\"`*"]
pub type CreateProcessMethod = i32;
#[doc = "*Required features: `\"Win32_System_WinRT_Shell\"`*"]
pub const CpCreateProcess: CreateProcessMethod = 0i32;
#[doc = "*Required features: `\"Win32_System_WinRT_Shell\"`*"]
pub const CpCreateProcessAsUser: CreateProcessMethod = 1i32;
#[doc = "*Required features: `\"Win32_System_WinRT_Shell\"`*"]
pub const CpAicLaunchAdminProcess: CreateProcessMethod = 2i32;
pub type IDDEInitializer = *mut ::core::ffi::c_void;
