// Copyright 2021 Contributors to the Parsec project.
// SPDX-License-Identifier: Apache-2.0
use crate::Context;

impl Context {
    // Missing function: AC_GetCapability
    // Missing function: AC_Send
    // Missing function: Policy_AC_SendSelect
}
